# -*- coding: utf-8 -*-

import datetime
from django.utils import timezone


def datetime_to_ecma262(value):
    """
    Convert a convencional datetime object
    to ecma-262 javascript/json format.
    """
    r = value.isoformat()
    if value.microsecond:
        r = r[:23] + r[26:]
    if r.endswith('+00:00'):
        r = r[:-6] + 'Z'
    return r


def ecma262_to_datetime(value):
    """
    Convert ecma-262 javascript format to
    correct python datetime format.

    This always return a timezone aware datetimes
    with utc as timezone.
    """

    spec = "%Y-%m-%dT%H:%M:%S.%f"
    is_utc = False

    if value.endswith("Z"):
        is_utc = True
        value = value[:-1]

    dt = strptime(value, spec)
    if is_utc:
        return timezone.make_aware(dt, timezone.utc)

    dt = timezone.make_aware(dt, timezone.get_current_timezone())
    dt = timezone.astimezone(timezone.utc)
    dt = timezone.utc.normalize(dt)
    return dt



# -*- coding: utf-8 -*-

from django.utils.datastructures import MergeDict


class RequestMergeDataMiddleware(object):
    """
    Inject request.DATA attribute containing a merge
    of GET, POST, and FILES attributes.

    Usefull for WTForms due it only accept one
    data parameter instead of two as django forms.
    """

    def process_request(self, request):
        request.DATA = MergeDict(request.GET,
                                 request.POST,
                                 request.FILES)



# -*- coding: utf-8 -*-

import datetime
import json

from django.core.serializers.json import DjangoJSONEncoder
from django.utils.functional import Promise
from django.utils import timezone

# Python3 compatibility
try:
    from django.utils.encoding import force_unicode as force_text
except ImportError:
    from django.utils.encoding import force_text


class LazyEncoder(DjangoJSONEncoder):
    """
    JSON encoder class for encode correctly traduction strings.
    Is for ajax response encode.
    """

    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        elif isinstance(obj, datetime.datetime):
            obj = timezone.localtime(obj)
        return super(LazyEncoder, self).default(obj)


def to_json(data, **kwargs):
    return json.dumps(data, cls=LazyEncoder, **kwargs)



# -*- coding: utf-8 -*-

"""
The various HTTP responses for use in returning proper HTTP codes.
"""

from django.http import HttpResponse


class HttpCreated(HttpResponse):
    status_code = 201

class HttpAccepted(HttpResponse):
    status_code = 202

class HttpNoContent(HttpResponse):
    status_code = 204

class HttpMultipleChoices(HttpResponse):
    status_code = 300

class HttpSeeOther(HttpResponse):
    status_code = 303

class HttpNotModified(HttpResponse):
    status_code = 304

class HttpBadRequest(HttpResponse):
    status_code = 400

class HttpUnauthorized(HttpResponse):
    status_code = 401

class HttpForbidden(HttpResponse):
    status_code = 403

class HttpNotFound(HttpResponse):
    status_code = 404

class HttpMethodNotAllowed(HttpResponse):
    status_code = 405

class HttpConflict(HttpResponse):
    status_code = 409

class HttpGone(HttpResponse):
    status_code = 410

class HttpTooManyRequests(HttpResponse):
    status_code = 429

class HttpApplicationError(HttpResponse):
    status_code = 500

class HttpNotImplemented(HttpResponse):
    status_code = 501





# -*- coding: utf-8 -*-

from django.views.generic import View
from django.template import RequestContext, loader
from django.shortcuts import render_to_response

from .. import http


class GenericView(View):
    response_cls = http.HttpResponse
    content_type = "text/html"

    def init(self, *args, **kwargs):
        pass

    def get_context_data(self):
        context = {"view": self}
        context.update(self.kwargs)
        return context

    def dispatch(self, request, *args, **kwargs):
        self.init(request, *args, **kwargs)
        return super(GenericView, self).dispatch(request, *args, **kwargs)

    def render(self, template=None, context=None, data=None,
               response_cls=None, content_type=None):

        output_data = data or b""

        if template:
            _context = self.get_context_data()
            _context.update(context or {})

            context_instance = RequestContext(self.request)
            output_data = loader.render_to_string(
                                template, _context,
                                context_instance=context_instance)

        if content_type is None:
            content_type = self.content_type

        if not response_cls:
            response_cls = self.response_cls

        return response_cls(output_data, content_type=content_type)


class GenericTemplateView(GenericView):
    tmpl_name = None

    def get(self, request, *args, **kwargs):
        if self.tmpl_name is None:
            raise ValueError("tmpl_name attr must be a valid template name")
        return self.render(self.tmpl_name)
